import json

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.store.models import *


class GmcDeleteDisapprovedProductsTask():
  def __init__(self, merchant_id, days_before_expire=14):
    self.merchant_id = merchant_id
    self.days_before_expire = days_before_expire
    self.gs = None
    self.gmc_store = GmcStore.get_or_none(GmcStore.merchant_id == merchant_id)
    if not self.gmc_store:
      return

    self.gs = GoogleShopping(merchant_id, json.loads(self.gmc_store.service_account_data))

  def run(self):
    if not self.gmc_store or not self.gs:
      return

    query = GmcProduct.select().join(GmcProductStatus, on=(GmcProduct.id == GmcProductStatus.gmc_product_id), attr='status').where(GmcProductStatus.item_disapproved == 1)
    gp_ids = []

    batch_size = 2500
    total_cnt = query.count()
    pages = int((total_cnt + batch_size) / batch_size)
    page = 1
    while page <= pages:
      gmc_ids = []
      for gp in query.paginate(page, batch_size):
        gp_ids.append(gp.id)
        gmc_ids.append(gp.gmc_id)
        if len(gmc_ids) < 250:
          continue

        self.gs.delete_products(gmc_ids)
        gmc_ids = []

      if gmc_ids:
        self.gs.delete_products(gmc_ids)

      page += 1

    chunks = [gp_ids[x:x + 1000] for x in range(0, len(gp_ids), 1000)]
    for chunk in chunks:
      GmcProductStatus.delete().where(GmcProductStatus.gmc_product_id.in_(chunk)).execute()
      GmcProduct.delete().where(GmcProduct.id.in_(chunk)).execute()
