# -*- coding: utf-8 -*-

import datetime
import pdb

from gspread.exceptions import APIError

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.utils.blacklist_loader import SheetBlacklistLoader
from em_tasks.store.models import *


class SheetBlacklistKeywordsSyncer():
  def __init__(self, service_account_data, spreadsheet_id):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)
    self.blacklist_loader = SheetBlacklistLoader(self.sheet_api, spreadsheet_id)

  def run(self):
    blacklist_keywords = self.blacklist_loader.load()
    if not blacklist_keywords:
      return

    keywords_mapping = dict()
    for blacklist_keyword in blacklist_keywords:
      keywords_mapping[blacklist_keyword['keywords'].strip().lower()] = blacklist_keyword

    keywords = list(keywords_mapping.keys())
    for bk in BlacklistKeyword.select().where(BlacklistKeyword.keywords.in_(keywords)).execute():
      keywords_mapping.pop(bk.keywords.strip().lower(), None)

    if not keywords_mapping:
      return

    BlacklistKeyword.insert_many(list(keywords_mapping.values())).execute()
