# -*- coding: utf-8 -*-

try:
  from urllib.parse import urlparse
except ImportError:
  from urlparse import urlparse

from em_tasks import logger

class BlacklistLoader():
  def load(self):
    raise NotImplementedError()


class SheetBlacklistLoader(BlacklistLoader):
  def __init__(self, sheet_api, spreadsheet_id):
    self.sheet_api = sheet_api
    self.spreadsheet_id = spreadsheet_id

  def load(self):
    spreadsheet = self.sheet_api.get_spreadsheet(self.spreadsheet_id)
    if not spreadsheet:
      logger.error('[SpreadSheetNotFound] %s', spreadsheet_id)
      return

    records = dict()
    worksheets = self.sheet_api.get_worksheets(spreadsheet)
    for worksheet in worksheets:
      name = worksheet.title.lower()
      if name != 'blacklist' and name != 'links':
        continue

      vals = self.sheet_api.get_worksheet_vals(worksheet)
      if name == 'links':
        for row in vals:
          url = row[0]
          if not url:
            continue

          parsed_url = urlparse(url)
          keywords = parsed_url.path.split('/').pop()
          records[keywords] = {
            'keywords': keywords,
            'match_type': 'Exact Match',
            'brand': None,
            'memo': None,
            'company': None
          }

        continue

      headers = None
      for row in vals:
        if not headers:
          if 'brand' not in row and 'Brand' not in row:
            continue

          headers = []
          for col in row:
            col_l = col.lower()
            if col_l.find('keywords') != -1:
              headers.append('keywords')
              continue

            if col_l.find('type') != -1:
              headers.append('match_type')
              continue

            if col_l.find('brand') != -1:
              headers.append('brand')
              continue

            if col_l.find('memo') != -1:
              headers.append('memo')
              continue

            if col_l.find('company') != -1:
              headers.append('company')
              continue

            headers.append(col_l)

          continue

        item = dict()
        for idx, header in enumerate(headers):
          try:
            item[header] = row[idx]
          except Exception as e:
            logger.exception(e)

        keywords = item['keywords'].strip()
        records[keywords] = {
          'keywords': keywords,
          'match_type': item['match_type'],
          'brand': item['brand'],
          'memo': item['memo'],
          'company': item['company']
        }

    return list(records.values())
