# -*- coding: utf-8 -*-

import datetime
import json
import pdb

from shipstation.api import *

from em_tasks import logger
from em_tasks.store.store_util import StoreUtil
from em_tasks.store.models import *


class ShipstationShipmentsSyncer():
  def __init__(self, shipstation_api_key, shipstation_api_password, days=14):
    self.shipstation_api_key = shipstation_api_key
    self.shipstation_api_password = shipstation_api_password
    self.client = ShipStation(key=shipstation_api_key, secret=shipstation_api_password)
    self.days = days

  def run(self):
    page_size = 100
    today = datetime.datetime.utcnow()
    from_date = today - datetime.timedelta(days=self.days)
    from_date_s = from_date.strftime('%Y-%m-%d')

    params = {
      'page': 1,
      'page_size': page_size,
      'ship_date_start': from_date_s
    }

    while True:
      res = self.client.fetch_shipments(params)
      if not res:
        break

      resp = res.json()
      if 'shipments' not in resp or not resp['shipments']:
        break

      self.save_shipstation_shipments(resp['shipments'])

      params['page'] += 1

  def save_shipstation_shipments(self, shipments):
    ssd = dict()
    for shipment in shipments:
      ssd[shipment['shipmentId']] = {
        'shipment_id': shipment['shipmentId'],
        'order_id': shipment['orderId'],
        'order_number': shipment['orderNumber'],
        'order_key': shipment['orderKey'],
        'create_date': shipment['createDate'],
        'ship_date': shipment['shipDate'],
        'shipment_cost': shipment['shipmentCost'],
        'insurance_cost': shipment['insuranceCost'],
        'tracking_number': shipment['trackingNumber'],
        'carrier_code': shipment['carrierCode'],
        'service_code': shipment['serviceCode'],
        'warehouse_id': shipment['warehouseId'],
        'shipment_items': json.dumps(shipment['shipmentItems'])
      }

    sos = dict()
    ss_mapping = dict()
    for ss in ssd.values():
      ss_mapping[ss['order_number']] = ss
    order_numbers = list(ss_mapping.keys())
    for so in ShipstationOrder.select().where(ShipstationOrder.order_number.in_(order_numbers)).execute():
      if so.order_number not in ss_mapping:
        continue

      ss = ss_mapping[so.order_number]
      sos[so.order_item_id] = ss

    order_item_ids = list(sos.keys())
    for order_item in OrderItem.select().where(OrderItem.id.in_(order_item_ids)).execute():
      if order_item.id not in sos:
        continue

      ss = sos[order_item.id]
      s_carrier = None
      if ss['service_code'].find('usps') != -1:
        s_carrier = 'USPS'
      elif ss['service_code'].find('ups') != -1:
        s_carrier = 'UPS'
      elif ss['service_code'].find('fedex') != -1:
        s_carrier = 'FEDEX'
      elif ss['service_code'].find('dhl') != -1:
        s_carrier = 'DHL'
      else:
        s_carrier = ss['service_code']
      s_tracking = ss['tracking_number']
      s_shipping_cost = round(float(ss['shipment_cost']) + float(ss['insurance_cost']), 2)
      if order_item.s_carrier != s_carrier or order_item.s_tracking != s_tracking:
        order_item.s_tracking = s_tracking
        order_item.s_shipping_cost = s_shipping_cost
        order_item.s_carrier = s_carrier
        order_item.save()

      order_event = OrderEvent.get_or_none(OrderEvent.order_item_id == order_item.id)
      if not order_event:
        order_event = OrderEvent()
        order_event.order_item_id = order_item.id
        order_event.event_type = 'OrderShipped'
        order_event.event_time = ss['createDate']
        order_event.event_data = json.dumps({'amount': s_shipping_cost})
        order_event.save()

    for ss in ShipstationShipment.select().where(ShipstationShipment.shipment_id.in_(list(ssd.keys()))).execute():
      ssd.pop(ss.shipment_id, None)

    if ssd:
      ShipstationShipment.insert_many(list(ssd.values())).execute()

    StoreUtil.update_order_items_status(order_item_ids)
